#
# CVSFile and CVSDirectory
#
# Files and directories managed with CVS
#
# Copyright (c) 2001-2003 by Philipp "philiKON" von Weitershausen
#
# This software is distributed under the terms of the Mozilla Public
# License (MPL)
#

"""
$Id: CVSObjects.py,v 1.3 2003/05/02 13:58:02 philipp Exp $
"""

from __future__ import nested_scopes
import os
import posixpath
from Persistence import Persistent

from interfaces.RevisionControlled import IRCFile, IRCDirectory

class CVSFile(Persistent):
    """
    A file managed with CVS
    """

    __implements__ = IRCFile

    __allow_access_to_unprotected_subobjects__ = 1

    isDirectory = 0

    def __init__(self, module_path, relative_path):
        # attributes defined in interface:
        self.filename = os.path.basename(relative_path)
        self.relative_path = relative_path
        self.absolute_path = os.path.join(module_path, relative_path)
        self._module_path = module_path

class CVSDirectory(Persistent):
    """
    A directory containing files managed with CVS
    """

    __implements__ = IRCDirectory

    __allow_access_to_unprotected_subobjects__ = 1

    isDirectory = 1

    def __init__(self, module_path, relative_path):
        # attributes defined in interface:
        self.relative_path = relative_path
        if relative_path == "":
            self.filename = os.path.basename(module_path)
            self.absolute_path = module_path
        else:
            self.filename = os.path.basename(relative_path)
            self.absolute_path = os.path.join(module_path, relative_path)

        self._module_path = module_path
        self._contents = {}

    def hasFile(self, filename):
        return filename in self.getDirectoryListing()
        
    def getFileObject(self, filename):
        if not self.hasFile(filename):
            raise ValueError, "File does not exist: %s" % filename
        
        if self._contents.has_key(filename):
            return self._contents[filename]
        
        abs_path = os.path.join(self.absolute_path, filename)
        rel_path = posixpath.join(self.relative_path, filename)

        if os.path.isfile(abs_path):
            obj = CVSFile(self._module_path, rel_path)
        elif os.path.isdir(abs_path) and filename != "CVS":
            obj = CVSDirectory(self._module_path, rel_path)
        else:
            return None

        self._contents[filename] = obj
        self._p_changed = 1
        return obj
    
    def getContents(self):
        objects = []
        contents = {}
        for filename in self.getDirectoryListing():
            obj = self.getFileObject(filename)
            objects.append(obj)
            contents[filename] = obj
        self._contents = contents
        return objects

    def getDirectoryListing(self, recursive=0):
        if not os.path.isdir(self.absolute_path):
            return []
        if not recursive:
            # filter dotted files and CVS
            filt_func = lambda path:not path.startswith('.') and path != "CVS"
            listing = os.listdir(self.absolute_path)
            listing = filter(filt_func, listing)
        else:
            def walker(rel_path=''):
                abs_path = os.path.join(self.absolute_path, rel_path)
                listing = os.listdir(abs_path)
                return_list = []
                for filename in listing:
                    abs = os.path.join(abs_path, filename)
                    rel = posixpath.join(rel_path, filename)
                    if os.path.isfile(abs) and not filename.startswith('.'):
                        return_list.append(rel)
                    elif os.path.isdir(abs) and filename != "CVS":
                        return_list += walker(rel)
                return return_list
            listing = walker()
        listing.sort()
        return listing

    def traverse(self, path, default=None):
        if path == self.relative_path:
            # it's me!
            return self
        elif self.hasFile(path):
            # I have it
            return self.getFileObject(path)
        else:
            segments = path.split('/')
            if not (segments and self.hasFile(segments[0])
                    and self.getFileObject(segments[0]).isDirectory):
                return default
            next = segments[0]
            segments = segments[1:]
            return self.getFileObject(next).traverse(posixpath.join(*segments))
